@@ -1,3 +1,5 @@
+from six import string_types
+
 from samtranslator.model.sqs import SQSQueue, SQSQueuePolicy, SQSQueuePolicies
 from samtranslator.model.exceptions import InvalidEventException
 
@@ -47,6 +49,11 @@ class EventBridgeRuleUtils:
         if dlq_queue_arn is not None:
             return dlq_queue_arn, []
         queue_logical_id = cw_event_source.DeadLetterConfig.get("QueueLogicalId")
+        if queue_logical_id is not None and not isinstance(queue_logical_id, string_types):
+            raise InvalidEventException(
+                cw_event_source.logical_id,
+                "QueueLogicalId must be a string",
+            )
         dlq_resources = EventBridgeRuleUtils.create_dead_letter_queue_with_policy(
             cw_event_source.logical_id, source_arn, queue_logical_id, attributes
         )
